@@ -978,8 +978,9 @@ Include "parser.h";
 	if(p_obj provides timer_order)
 		_order = p_obj.timer_order;
 	for(_i=_i-1 : _i>=0 : _i--) {
-		the_timers-->(_i+1) = the_timers-->_i;
-		_obj = the_timers-->_i & ~WORD_HIGHBIT;
+		_obj = the_timers-->_i;
+		the_timers-->(_i+1) = _obj;
+		_obj = _obj & ~WORD_HIGHBIT;
 		_order2 = 100;
 		if(_obj provides timer_order)
 			_order2 = _obj.timer_order;
@@ -1320,10 +1321,13 @@ Object thedark "Darkness"
 #Ifndef OPTIONAL_NO_DARKNESS;
 	_UpdateDarkness(true);
 #Endif;
-	if(update_moved) {
+
+	if(update_moved || child(player) ~= last_player_child or 0) {
 		_NoteObjectAcquisitions();
 		update_moved = false;
+		last_player_child = child(player);
 	}
+
 ];
 
 #Ifdef OPTIONAL_PROVIDE_UNDO_FINAL;
@@ -1510,13 +1514,13 @@ Object thedark "Darkness"
 		_parser_oops = parser_unknown_noun_found;
 .do_it_again;
 #Ifdef DEBUG_TIMER;
-	timer1 = 0-->2;
+		timer1 = 0-->2;
 #Endif;
 		_sentencelength = _ParseAndPerformAction();
 #Ifdef DEBUG_TIMER;
-	timer1 = 0-->2 - timer1;
-	print "[ParseAndPerformAction took ",timer1," jiffies]^";
-	timer1 = 0-->2;
+		timer1 = 0-->2 - timer1;
+		print "[ParseAndPerformAction took ",timer1," jiffies]^";
+		timer1 = 0-->2;
 #Endif;
 		if(action == ##OopsCorrection) {
 			if(_again_saved && _parser_oops > 0) {
@@ -1544,6 +1548,8 @@ Object thedark "Darkness"
 			} else {
 				PrintMsg(MSG_PARSER_NOTHING_TO_AGAIN);
 			}
+		} else if(parse->1 == 0) {
+			_again_saved = 0;
 		} else {
 			! store the current buffer to 'again'
 			_again_saved = true;
